% Funci dibuix_carregabarra per a Clcul Matricial d'Estructures 2 (CME2).
%
% Per Sergio Guerrero Miralles, darrera versi 13 d'octubre de 2.012.
%
% Crea ojectes grfics del tipus hgtransform amb el dibuix de les
% crregues puntuals i moments aplicats a les barres, segons la vista.
% Retorna els handles ja rotats i orientats segons la vista, per a
% collocar i escalar a la grafica.
%
% Entrada:
% bar: objecte tipus barra, estructura de dades amb les propietats i les crregues de la barra.
% vectorbarra: vector 1x3 amb la direcci de la barra.
% vista: tipus de vista en format cadena de texte, 'xy','yz','xz' o '3d'.
% fontcarregues: tipus de font del texte.
% tamanyfontcarregues: tamany de la font.
%
% Sortida:
% puntual: handle de l'objecte grfic hgtransform creat que representa la crrega puntual a la barra.
% moment: handle de l'objecte grfic hgtransform creat que representa el moment a la barra.
%
% Funcionament:
% Es parteix de les coordenades dels punts per dibuixar una fletxa al plnol xy amb direcci i sentit + x.
% Es rota la projecci de la direcci de la crrega sobre el plnol de la vista al plnol xy, per treballar-hi al plnol xy.
% Es roten els punts segons la orientaci de la projecci de la carrega.
% Es roten els punts de nou al plnol de la vista corresponent i es dibuixa la fletxa.
% De manera idntica per als moments.
% No dibuixa les forces al plnol perpendicular al de la vista, s els moments.

%% FUNCI PRINCIPAL:

function [puntual,moment] = dibuix_carregabarra(bar,vectorbarra,vista,fontcarregues,tamanyfontcarregues,fmax)
    bar.puntual(2:4)=bar.puntual(1)*bar.puntual(2:4)/norm(bar.puntual(2:4)); % Crrega puntual en forma de vector.
    bar.moment(2:4)=bar.moment(1)*bar.moment(2:4)/norm(bar.moment(2:4)); % Moment en forma de vector.
    if any(bar.puntual(2:4)) % Si hi ha crrega puntual.
        puntual=hgtransform('HitTestArea','on','Clipping','on','UserData',bar.num);
        switch vista
            case 'xy'
                dibuixa_forca(puntual,bar.puntual(1),[bar.puntual(2:3),0],vista,fmax); % Dibuixa la fletxa representant la fora al plnol xy.
                postxt=[-0.7*bar.puntual(2:3)/norm(bar.puntual(2:3)),0];
            case 'xz'
                dibuixa_forca(puntual,bar.puntual(1),[bar.puntual(2),0,bar.puntual(4)],vista,fmax); % Dibuixa la fletxa representant la fora al plnol xz.
                postxt=-0.7*[bar.puntual(2),0,bar.puntual(4)]/norm([bar.puntual(2),0,bar.puntual(4)]);
            case 'yz'
                dibuixa_forca(puntual,bar.puntual(1),[0,bar.puntual(3:4)],vista,fmax); % Dibuixa la fletxa representant la fora al plnol yz.
                postxt=[0,-0.7*bar.puntual(3:4)/norm(bar.puntual(3:4))];
            case '3d'
                dibuixa_forca(puntual,bar.puntual(1),bar.puntual(2:4),vista,fmax); % Dibuixa la fletxa representant la fora a l'espai.
                postxt=-0.7*bar.puntual(2:4)/norm(bar.puntual(2:4));
        end
        texte_carrega(puntual,postxt,strcat('P=',num2str(bar.puntual(1))),fontcarregues,tamanyfontcarregues) % Texte amb la magnitud de la fora.
    else
        puntual=[];
    end

    if any(bar.moment(2:4)) % Si hi ha moment.
        moment=hgtransform('HitTestArea','on','Clipping','on','UserData',bar.num);
        switch vista
            case 'xy'
                postxt=dibuixa_moment(moment,[bar.moment(2:3),0],bar.moment(4),[vectorbarra(1:2),0],vista); % Dibuixa la representaci del moment al plnol xy.
            case 'xz'
                postxt=dibuixa_moment(moment,[bar.moment(2),0,bar.moment(4)],bar.moment(3),[vectorbarra(1),0,vectorbarra(3)],vista); % Dibuixa la representaci del moment al plnol xz.
            case 'yz'
                postxt=dibuixa_moment(moment,[0,bar.moment(3:4)],bar.moment(2),[0,vectorbarra(2:3)],vista); % Dibuixa la representaci del moment al plnol yz.
            case '3d'
                postxt=dibuixa_moment_fletxa(moment,bar.moment(2:4),vista); % Dibuixa la representaci del moment a l'espai.
        end
        texte_carrega(moment,postxt,strcat('M=',num2str(bar.moment(1))),fontcarregues,tamanyfontcarregues) % Texte amb la magnitud del moment.
    else
        moment=[];
    end
end

%% FUNCIONS PER DIBUIXAR FORCES I MOMENTS:

function dibuixa_forca(handle,p,direccio,vista,fmax) % Dibuixa la fletxa que representa la fora aplicada a la barra.
    prop=p/fmax;
    XYZlf=[-prop-0.23,0;0,0;0,0]; % Coordenades x,y,z de la recta de la fletxa en horitzontal i cap a la dreta.
    XYZpf=[0,-0.23,-0.23;0,-0.0375,0.0375;0,0,0]; % Coordenades x,y,z de la punta de fletxa en horitzontal i cap a la dreta.
    dibuixa_fletxa3d(XYZlf,XYZpf,handle,direccio,vista); % Dibuixa la fletxa orientant-la en la direccio de la fora.
end

function postxt=dibuixa_moment(handle,momentpla,momenttransversal,projbarraalpla,vista) % Dibuixa el moment, arc per al moment transversal i fletxa per als moments al pla.
    if momenttransversal % Si hi ha moment en z.
        postxt=dibuixa_moment_arc(handle,momenttransversal,projbarraalpla,vista); % Dibuixa l'arc del moment.
    end
    if any(momentpla) % Si hi ha moment en x o y.
        postxt=dibuixa_moment_fletxa(handle,momentpla,vista); % Representa el moment amb una fletxa de dos puntes.
    end
end

function postxt=dibuixa_moment_fletxa(pare,direccio,vista) % Representa el moment amb una fletxa de dos puntes.
    XYZlmf=[0,0.5;0,0;0,0]; % Coordenades x,y,z de la recta de la fletxa en hortitzontal i cap a la dreta.
    XYZpmf=[0.5,0.425,0.425,0.35,0.35,0.425,0.425;0,-0.0375,0,-0.0375,0.0375,0,0.0375;0,0,0,0,0,0,0]; % Coordenades x,y,z de la punta de fletxa doble en horitzontal i cap a la dreta.
    postxt=dibuixa_fletxa3d(XYZlmf,XYZpmf,pare,direccio,vista); % Dibuixa el moment orientant-lo en la direcci.
end

function postxt=dibuixa_moment_arc(handle,modul,direccio,vista)
    if ~any(direccio)
        direccio=[1,0,0];
    end
    signe=abs(modul)/modul; % Signe del moment.
    kk=0.3*pi:0.05*pi:1.15*pi; % Arc de moment.
    XYZlm=[0,cos(kk)*0.3;0,sin(kk)*0.3*signe;0,kk*0]; % Coordenades x,y,z de l'arc.
    XYZpm=[cos(1.4*pi)*0.3,cos(1.15*pi)*0.3-cos(5*pi/18)*0.0375,cos(1.15*pi)*0.3+cos(5*pi/18)*0.0375;sin(1.4*pi)*0.3*signe,sin(1.15*pi)*0.3*signe-sin(5*pi/18)*0.0375*signe,sin(1.15*pi)*0.3*signe+sin(5*pi/18)*0.0375*signe;0,0,0]; % Coordenades x,y,z de la punta de fletxa.
    postxt=dibuixa_fletxa3d(XYZlm,XYZpm,handle,direccio,vista); % Dibuixa el moment orientant-lo en la direcci.
end

function postxt=dibuixa_fletxa3d(XYZl,XYZp,handle,direccio,vista)
    % Dibuixa una fletxa, formada per una lnia i una punta de fletxa
    % definides per les coordenades introdudes i orientades segons el vector direcci.
    % Estableix la posici del texte a la punta de la fletxa.
    
    switch vista
        case 'xz'
            m=[1,0,0;0,0,1;0,-1,0]; % Matriu canvi de base de (x,y,z) a (x,z,-y).
        case 'yz'
            m=[0,0,1;1,0,0;0,1,0]; % Matriu canvi de base de (x,y,z) a (x,z,-y).
        otherwise
            m=[1,0,0;0,1,0;0,0,1]; % Matriu identitat.
    end
    direccio=(m'*direccio')'; % Rota la direcci al plnol xy.
    mat=calcul_matriu(direccio); % Matriu de gir al plnol xy.
    XYZl=m*mat*XYZl; % Gira la fletxa o l'arc del moment seguint la inclinaci de la barra i torna al plnol de la vista.
    XYZp=m*mat*XYZp;
    
    line(XYZl(1,:),XYZl(2,:),XYZl(3,:),'color','k','LineWidth',2,'Parent',handle,'HitTest','off');
    patch(XYZp(1,:),XYZp(2,:),XYZp(3,:),'k','Parent',handle,'HitTest','off');
    
    postxt=XYZp(:,1)'*1.25;
end

function texte_carrega(handle,coords,txt,fontcarregues,tamanyfontcarregues) % Texte amb la magnitud de la fora.
    text(coords(1),coords(2),coords(3),txt,'VerticalAlignment','middle','Parent',handle,'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'HitTest','off')
end

%% CLCUL DE LA MATRIU DE CANVI DE BASE:

function matriu_rotacio=calcul_matriu(vector)
    vector=vector/norm(vector); % Normaliza el vector.
    if abs(vector(3))==1 % Si el vector s parallel a l'eix z...
        vector3=[-vector(3),0,0];
        vector2=[0,1,0];
    else
        vector2=cross([0,0,1],vector); % Calcula dos vectors perpendiculars al primer i entre ells per crear la matriu de canvi de base.
        vector2=vector2/norm(vector2);
        vector3=cross(vector,vector2);
        vector3=vector3/norm(vector3);
    end
    matriu_rotacio=[vector',vector2',vector3'];
end