% Funci dibuix_carreganus per a Clcul Matricial d'Estructures 2 (CME2).
%
% Per Sergio Guerrero Miralles, darrera versi 13 d'octubre de 2.012.
%
% Crea ojectes grfics del tipus hgtransform amb el dibuix de les
% crregues puntuals i moments aplicats als nusos o les reaccions.
% Retorna el handle, per a collocar i escalar a la grfica.
% Crregues independents per a cada eix.
%
% Entrada:
% - n: nmero del nus.
% - posici: posici del nus a l'espai.
% - centre: mitja de la posici de tots els nusos.
% - forces: vector de forces aplicades al nus.
% - moments: vector de moments aplicats al nus.
% - vista: 'xy','xz',yz' o '3d' per a la seva correcta representaci.
% - fontcarregues: font dels textos de les crregues.
% - tamanyfontcarregues: tamany de la font;
%
% Sortida:
% - carrega: handle de l'objecte hgtransform amb la representaci de les crregues.
%
% Funcionament:
% Es parteix de les coordenades dels punts per dibuixar una fletxa al plnol xy amb direcci i sentit + x.
% Es treballa al plnol xy com si fs el de  vista.
% Es roten els punts al plnol de la vista corresponent i es dibuixa la fletxa.
% De manera idntica per als moments.
% No dibuixa les forces al plnol perpendicular al de la vista, s els moments.

function hgcarrega = dibuix_carreganus(n,posicio,centre,forces,moments,vista,fontcarregues,tamanyfontcarregues,fmax)

    tol=1e-8; % Valor mnim, per evitar mostrar errors d'arrodoniment a 0.

    if any(forces) || any(moments)
        hgcarrega=hgtransform('HitTestArea','on','Clipping','on','UserData',n); % Crea l'objecte hgtransform.

        switch vista
            case 'xy'
                dibuixa_forces(hgcarrega,tol,forces(1:2),posicio(1:2)-centre(1:2),fontcarregues,tamanyfontcarregues,[1,0,0;0,1,0;0,0,1],{'Fx','Fy'},fmax) % Dibuixa les forces al plnol xy.
                dibuixa_moments(hgcarrega,tol,moments(1:2),moments(3),posicio(1:2)-centre(1:2),fontcarregues,tamanyfontcarregues,[1,0,0;0,1,0;0,0,1],{'Mx','My','Mz'}) % Dibuixa els moments al plnol xy.
            case 'xz'
                dibuixa_forces(hgcarrega,tol,[forces(1),forces(3)],[posicio(1)-centre(1),posicio(3)-centre(3)],fontcarregues,tamanyfontcarregues,[1,0,0;0,0,-1;0,1,0],{'Fx','Fz'},fmax) % Dibuixa les forces al plnol xz.
                dibuixa_moments(hgcarrega,tol,[moments(1),moments(3)],moments(2),[posicio(1)-centre(1),posicio(3)-centre(3)],fontcarregues,tamanyfontcarregues,[1,0,0;0,0,-1;0,1,0],{'Mx','Mz','My'}) % Dibuixa els moments al plnol xz.
            case 'yz'
                dibuixa_forces(hgcarrega,tol,forces(2:3),posicio(2:3)-centre(2:3),fontcarregues,tamanyfontcarregues,[0,0,1;1,0,0;0,1,0],{'Fy','Fz'},fmax) % Dibuixa les forces al plnol yz.
                dibuixa_moments(hgcarrega,tol,moments(2:3),moments(1),posicio(2:3)-centre(2:3),fontcarregues,tamanyfontcarregues,[0,0,1;1,0,0;0,1,0],{'My','Mz','Mx'}) % Dibuixa els moments al plnol yz.
            case '3d'
                dibuixa_forces(hgcarrega,tol,forces(1:3),posicio(1:3)-centre(1:3),fontcarregues,tamanyfontcarregues,[1,0,0;0,1,0;0,0,1],{'Fx','Fy','Fz'},fmax) % Dibuixa les forces a l'espai.
                dibuixa_moments(hgcarrega,tol,moments(1:3),0,0,fontcarregues,tamanyfontcarregues,[1,0,0;0,1,0;0,0,1],{'Mx','My','Mz'}) % Dibuixa els moments a l'espai.
        end
    else
        hgcarrega=[];
    end
end

function dibuixa_forces(handle,tol,forcesxy,posxy,fontcarregues,tamanyfontcarregues,matriu_canvi_vista,txt,fmax)
    % Dibuixa les forces al plnol xy i desprs aplica una matriu de rotaci per adaptar-les al plnol de visualitzaci.
    if abs(forcesxy(1))>tol % Forces en x.
        [postxt,halin]=dibuixa_forca(handle,forcesxy(1),posxy(1),matriu_canvi_vista,fmax);
        text(postxt(1),postxt(2),postxt(3),strcat(txt(1),'=',num2str(forcesxy(1))),'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'Parent',handle,'HorizontalAlignment',halin)
    end
    if abs(forcesxy(2))>tol % Forces en y.
        [postxt,halin]=dibuixa_forca(handle,forcesxy(2),posxy(2),matriu_canvi_vista*[0,1,0;1,0,0;0,0,1],fmax);
        text(postxt(1),postxt(2),postxt(3),strcat(txt(2),'=',num2str(forcesxy(2))),'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'Parent',handle,'HorizontalAlignment',halin)
    end
    if numel(forcesxy)==3 && abs(forcesxy(3))>tol % Forces en z.
        [postxt,halin]=dibuixa_forca(handle,forcesxy(3),posxy(3),matriu_canvi_vista*[0,1,0;0,0,1;1,0,0],fmax);
        text(postxt(1),postxt(2),postxt(3),strcat(txt(3),'=',num2str(forcesxy(3))),'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'Parent',handle,'HorizontalAlignment',halin)
    end
end

function [postxt,halin]=dibuixa_forca(handle,forca,pos,mat,fmax)
    % Fletxa definida al plnol xy seguint la direcci de l'eix x, s'aplica una matriu de rotaci per situar-la a l'eix corresponent.
    prop=abs(forca/fmax); % Longitud de la fletxa, proporcional a la fora.
    signe=forca/abs(forca); % Sentit de la fora.
    pos=pos/abs(pos); % Posici rellativa de la fletxa, >0 dreta, <0 esquerra

    if pos*signe>0 % estirant
        XYlf=[prop+0.33,0.1;0,0]*signe; % Coordenades de la lnia de la fletxa que representa la fora.
        XYpf=[0.1+prop,0.33+prop,0.1+prop;.0375,0,-.0375]*signe; % Coordenades de la punta de fletxa que representa la fora.
        if signe>0
            halin='left';
        else
            halin='right';
        end
    else % empenyent
        XYlf=-[prop+0.33,0.33;0,0]*signe; % Coordenades de la lnia de la fletxa que representa la fora.
        XYpf=[-0.33,-0.1,-0.33;.0375,0,-.0375]*signe; % Coordenades de la punta de fletxa que representa la fora.
        if signe>0
            halin='right';
        else
            halin='left';
        end
    end
    XYlf=mat*[XYlf;0,0];
    XYpf=mat*[XYpf;0,0,0];
    line(XYlf(1,:),XYlf(2,:),XYlf(3,:),'color','k','LineWidth',2,'Parent',handle,'HitTest','off');
    patch(XYpf(1,:),XYpf(2,:),XYpf(3,:),'k','Parent',handle,'HitTest','off');
    postxt=XYlf(:,1)*1.1;
end

function dibuixa_moments(handle,tol,momentsxy,momentz,posxy,fontcarregues,tamanyfontcarregues,matriu_canvi_vista,txt)
    % Dibuixa els moments al plnol xy i desprs aplica una matriu de rotaci per adaptar-los al plnol de visualitzaci.
    if abs(momentsxy(1))>tol % Moment en x. Fletxa de dos puntes al plnol xy.
        [postxt,halin,valin]=dibuixa_moment_fletxa(handle,momentsxy(1),matriu_canvi_vista);
        text(postxt(1),postxt(2),postxt(3),strcat(txt(1),'=',num2str(momentsxy(1))),'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'Parent',handle,'HorizontalAlignment',halin,'VerticalAlignment',valin)
    end
    if abs(momentsxy(2))>tol % Moment en y. Fletxa de dos puntes al plnol xy.
        [postxt,halin,valin]=dibuixa_moment_fletxa(handle,momentsxy(2),matriu_canvi_vista*[0,1,0;1,0,0;0,0,1]);
        text(postxt(1),postxt(2),postxt(3),strcat(txt(2),'=',num2str(momentsxy(2))),'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'Parent',handle,'HorizontalAlignment',halin,'VerticalAlignment',valin)
    end
    if numel(momentsxy)==3 && abs(momentsxy(3))>tol % Moment en z. Fletxa de dos puntes si es representa a l'espai.
        [postxt,halin,valin]=dibuixa_moment_fletxa(handle,momentsxy(3),matriu_canvi_vista*[0,1,0;0,0,1;1,0,0]);
        text(postxt(1),postxt(2),postxt(3),strcat(txt(3),'=',num2str(momentsxy(3))),'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'Parent',handle,'HorizontalAlignment',halin,'VerticalAlignment',valin)
    elseif abs(momentz)>tol % Momennt en z. Arc si es representa al plnol xy.
        [postxt,halin,valin]=dibuixa_moment_arc(handle,momentz,posxy,matriu_canvi_vista);
        text(postxt(1),postxt(2),postxt(3),strcat(txt(3),'=',num2str(momentz)),'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'Parent',handle,'HorizontalAlignment',halin,'VerticalAlignment',valin)
    end
    
end

function [postxt,halin,valin]=dibuixa_moment_fletxa(handle,moment,mat)
    % Fletxa definida al plnol xy seguint la direcci de l'eix x, s'aplica una matriu de rotaci per situar-la a l'eix corresponent.
    signe=moment/abs(moment); % Signe del moment.
    XYZlmf=mat*[.5*signe,0;0,0;0,0]; % Coordenades de la lnia de la fletxa que representa el moment.
    XYZpmf=mat*[[.425,.5,.425,.425,.35,.35,.425]*signe;.0375,0,-.0375,0,-.0375,.0375,0;0,0,0,0,0,0,0]; % Coordenades de la punta de la fletxa que representa el moment.

    line(XYZlmf(1,:),XYZlmf(2,:),XYZlmf(3,:),'color','k','LineWidth',2,'Parent',handle,'HitTest','off'); % Dibuixa la lnia.
    patch(XYZpmf(1,:),XYZpmf(2,:),XYZpmf(3,:),'k','Parent',handle,'HitTest','off'); % Dibuixa la punta de fletxa.
    postxt=XYZlmf(:,1)*1.1; % Posiciona el texte.
    if signe >0 % Defineix l'alineaci del texte.
        halin='left';
        valin='bottom';
    else
        halin='right';
        valin='top';
    end
end

function [postxt,halin,valin]=dibuixa_moment_arc(handle,momentz,posxy,matriu_canvi_vista)
    signe=momentz/abs(momentz); % Signe del moment.
    if matriu_canvi_vista(2,3)==-1 % Correcci direcci moments en xz, en aquest plnol el moment positiu s horari.
        signe=-signe;
    end
    % Gira l'arc segons la seva posici al plnol aplicant una matriu de rotaci sobre l'eix z.
    if posxy(1)>0 % dreta
        if posxy(2)>0 %dalt
            mat=matriu_canvi_vista;
            valin='bottom';
        else % baix
            mat=matriu_canvi_vista*[0,1,0;-1,0,0;0,0,1];
            valin='top';
        end
        halin='left';
    else % esquerra
        if posxy(2)>0 % dalt
            mat=matriu_canvi_vista*[0,-1,0;1,0,0;0,0,1];
            valin='bottom';
        else %baix
            mat=matriu_canvi_vista*[-1,0,0;0,-1,0;0,0,1];
            valin='top';
        end
        halin='right';
    end
   
    kk=0.1*pi:pi/20:0.65*pi; % Punts de l'arc.
    XYZlm=mat*[cos(kk)*0.3*signe;sin(kk)*0.3+0.15;kk*0]; % Defineix la lnia de l'arc.
    XYZpm=mat*[[sin(1.15*pi)*.3+sin(5*pi/18)*.0375,sin(1.4*pi)*.3,sin(1.15*pi)*.3-sin(5*pi/18)*.0375]*signe;-cos(1.15*pi)*.3-cos(5*pi/18)*.0375+.15,-cos(1.4*pi)*.3+.15,-cos(1.15*pi)*.3+cos(5*pi/18)*.0375+.15;0,0,0]; % Punta de l'arc.
    line(XYZlm(1,:),XYZlm(2,:),XYZlm(3,:),'color','k','LineWidth',2,'Parent',handle,'HitTest','off'); % Dibuixa la lnia.
    patch(XYZpm(1,:),XYZpm(2,:),XYZpm(3,:),'k','Parent',handle,'HitTest','off'); % Dibuixa la punta.
    postxt=mat*[.4;.4;0]; % Posiciona el texte a l'extrem superior dret de l'arc original.
end