% Funci dibuix_diagrames per a Clcul Matricial d'Estructures 2 (CME2).
%
% Per Sergio Guerrero Miralles, darrera versi 1 de juny de 2.012.
%
% Darrera versi 13/10/2012
%
% Dibuixa els diagrames d'esforos i deformada de les barres.
% Crea un objecte diagrama per a cada barra, amb un objecte hgtransform per a cada tipus de diagrama.
% Tot en eixos locals per a ser traslladat i rotat al seu lloc a la grfica.
%
% Calcula els trams i les expressions dels diferents diagrames.
% Calcula la deformada.
% Calcula mxims i mnims.
% Dibuixa els diagrames i la deformada.
% Retorna les expressions en forma de texte.
%
% Entrada:
% - Long: longitud de la barra.
% - esforcos: esforos en extrems de barra, en format vector 1x12.
% - moviments: moviments dels nusos inferior i superior, en format vector 1x12.
% - repartida: crrega repartida, vector 1x6. [mdul,x,y,z,distncia a nus inferior,distncia a nus superior].
% - projectada: crrega repartida projectada, mateix format que la repartida.
% - puntual: crrega puntual, vector 1x5. [mdul,x,y,z,distancia nus inferior].
% - moment: crrega moment, vecotr 1x5, mateix format que puntual.
% - E,A,Iz,Iz: propietats mecniques de la barra.
%
% Sortida:
% - diag: objecte classe diagrama, on:
% + hgN: handle de l'objecte hgtransform que cont la grfica del diagrama d'axils, en eixos locals de la barra.
% + hgTy: handle de l'objecte hgtransform que cont la grfica del diagrama de tallants en y, en eixos locals de la barra.
% + hgTz: handle de l'objecte hgtransform que cont la grfica del diagrama de tallants en z, en eixos locals de la barra.
% + hgMx: handle de l'objecte hgtransform que cont la grfica del diagrama de moments en x, en eixos locals de la barra.
% + hgMy: handle de l'objecte hgtransform que cont la grfica del diagrama de moments en y, en eixos locals de la barra.
% + hgMz: handle de l'objecte hgtransform que cont la grfica del diagrama de moments en z, en eixos locals de la barra.
% + hgdeformada: handle de l'objecte hgtransform que cont la grafica de la deformada, en eixos locals de la barra.
% + Tmax: Tallant mxim en valor absolut, per a escalar els diagrames d'axils i tallants a la grfica.
% + Mmax: Tallant mxim en valor absolut, per a escalar els diagrames de moments a la grafica.
% + Dmax: Desplaament mxim en valor absolut, per a escalar la deformada a la grafica.
% + Oini: Gir a l'extrem inicial de la barra.
% + Ofin: Gir a l'extrem final de la barra.
% + fletxay: [fletxa mxima en y,ndex a la llista de coordenades de la deformada] 
% + fletxaz: [fletxa mxima en z,ndex a la llista de coordenades de la deformada] 
% - varargout: Texte amb les expressions analtiques de les lleis d'esforos, deformada i mxims i mnims. Opcional.
%
% Funcionament:
% Calcula les lleis d'esforos a partir de les carregues i els eforos en extrem de barra, en forma
% de coeficients de polinomi per a cada tram de barra delimitat per la localitzaci de les carregues.
% Per obtenir la deformada es calculen les integrals de les lleis d'esforos a cada tram. S'utilitza
% com a condici de contorn els desplaaments verticals calculats dels nusos extrems i la continutat
% de la deformada i la seva pendent.
%

%% FUNCI PRINCIPAL:

function [diag,varargout]=dibuix_diagrames(long,esforcos,moviments,repartida,projectada,puntual,moment,E,A,Iz,Iy)

detall=nargout-1; % Si s'hi introdueix ms d'una sortida retorna els diagrames amb marques i dades numriques per a la presentaci en detall. Amb una sla sortida dibuixa els diagrames simples per a ser presentats a la grafica de l'estructura.

sep=long/50; % Separaci entre punts per al dibuix dels diagrames, 'resoluci' del diagrama.
diag=diagrama; % Crea l'objecte de la classe diagrama.

[puntual,moment,repartida,projectada,dcx]=carregues(puntual,moment,repartida,projectada); % Normalitza les crregues per al clcul.


N=calcul_tallant(esforcos(1),[puntual(2),puntual(5)],[repartida(2),repartida(5),long-repartida(6)],[projectada(2),projectada(5),long-projectada(6)],long,dcx); % Expressi axils.
Ty=calcul_tallant(esforcos(2),[puntual(3),puntual(5)],[repartida(3),repartida(5),long-repartida(6)],[projectada(3),projectada(5),long-projectada(6)],long,dcx); % Expressi tallants en y.
Tz=calcul_tallant(esforcos(3),[puntual(4),puntual(5)],[repartida(4),repartida(5),long-repartida(6)],[projectada(4),projectada(5),long-projectada(6)],long,dcx); % Expressi tallants en z.

Mx=calcul_moment(0,esforcos(4),[moment(2),moment(5)],[0,0],[0,0,0],[0,0,0],long,dcx,1); % Expressi moment torsor.
My=calcul_moment(esforcos(3),esforcos(5),[moment(3),moment(5)],[puntual(4),puntual(5)],[repartida(4),repartida(5),long-repartida(6)],[projectada(4),projectada(5),long-projectada(6)],long,dcx,-1); % Expressi moment en y.
Mz=calcul_moment(esforcos(2),esforcos(6),[moment(4),moment(5)],[puntual(3),puntual(5)],[repartida(3),repartida(5),long-repartida(6)],[projectada(3),projectada(5),long-projectada(6)],long,dcx,1); % Expressi moment en z.

dx=calcul_dx(moviments(1),N,E*A,long); % Expressi deformaci longitudinal.
[dy,Oz,Ozini,Ozfin]=calcul_d([moviments(2),moviments(8)],Mz,E*Iz,long,1); % Expressions deformaci transversal en y i gir en z.
[dz,Oy,Oyini,Oyfin]=calcul_d([moviments(3),moviments(9)],My,E*Iy,long,-1); % Expressions deformaci transversal en z i gir en y.
diag.Oini=[Oyini,Ozini];
diag.Ofin=[Oyfin,Ozfin];

N0=zero(N); % Punts on N=0.
Ty0=zero(Ty); % Punts on Ty=0.
Tz0=zero(Tz); % Punts on Tz=0.
Mx0=zero(Mx); % Punts on Mx=0.
My0=zero(My); % Punts on My=0.
Mz0=zero(Mz); % Punts on Mz=0.
Oy0=zero(Oy); % punts on Oy=0.
Oz0=zero(Oz); % punts on Oz=0.

[diag.hgN,Nmin,Nmax]=dibudiag('N',N,N0,[],sep,long,1,1); % Dibuixa el diagrama d'axils.
[diag.hgTy,Tymin,Tymax]=dibudiag('Ty',Ty,Ty0,[],sep,long,1,1); % Dibuixa el diagrama de tallants en y.
[diag.hgTz,Tzmin,Tzmax]=dibudiag('Tz',Tz,Tz0,[],sep,long,1,1); % Dibuixa el diagrama de tallants en z.
[diag.hgMx,Mxmin,Mxmax]=dibudiag('Mx',Mx,Mx0,[],sep,long,1,1); % Dibuixa el diagrama de moments en x.
[diag.hgMy,Mymin,Mymax]=dibudiag('My',My,My0,Tz0,sep,long,1,-1); % Dibuixa el diagrama de moments en y.
[diag.hgMz,Mzmin,Mzmax]=dibudiag('Mz',Mz,Mz0,Ty0,sep,long,1,1); % Dibuixa el diagrama de moments en z.
[diag.hgdeformada,dxmax,dymax,dzmax]=dibudef(dx,dy,dz,My0,Mz0,Oy0,Oz0,Oyini,Oyfin,Ozini,Ozfin,sep,long,detall); % Dibuixa la deformada.

diag.Tmax=max(abs([Nmin(2),Nmax(2),Tymin(2),Tymax(2),Tzmin(2),Tzmax(2)])); % Mxim tallant o axil en termes absoluts, per a determinar l'escala dels diagrames.
diag.Mmax=max(abs([Mxmin(2),Mxmax(2),Mymin(2),Mymax(2),Mzmin(2),Mzmax(2)])); % Mxim moment en termes absoluts, per a determinar l'escala dels diagrames.
diag.Dmax=max(abs([dxmax(2),dymax(2),dzmax(2)])); % Mxim desplaament en termes absoluts, per a determinar l'escala de la deformada.


if detall
    txt=monta_texte('','N',equac(N),Nmin,Nmax,N0); % Crea les cadenes de texte amb les expressions analtiques.
    txt=monta_texte(txt,'Ty',equac(Ty),Tymin,Tymax,Ty0);
    txt=monta_texte(txt,'Tz',equac(Tz),Tzmin,Tzmax,Tz0);
    txt=monta_texte(txt,'Mx',equac(Mx),Mxmin,Mxmax,Mx0);
    txt=monta_texte(txt,'My',equac(My),Mymin,Mymax,My0);
    txt=monta_texte(txt,'Mz',equac(Mz),Mzmin,Mzmax,Mz0);
    txt=monta_texte(txt,'y',equac(Oy));
    txt=monta_texte(txt,'z',equac(Oz));
    txt=monta_texte(txt,'dx',equac(dx),dxmax);
    txt=monta_texte(txt,'dy',equac(dy),dymax);
    txt=monta_texte(txt,'dz',equac(dz),dzmax);
    varargout{1}=char(txt);
end
end



%% NORMALITZACI DE LES CRREGUES.
%
% Multiplica el vector direcci de cada crrega pel seu mdul.
% Obt la rellaci entre longitud de la crrega projectada i de la barra.

function [puntual,moment,repartida,projectada,dcx]=carregues(puntual,moment,repartida,projectada)
    if puntual(1) && any(puntual(2:4))
        puntual(2:4)=puntual(1)*puntual(2:4)/norm(puntual(2:4));
    else
        puntual(2:4)=[0,0,0];
    end
    if moment(1) && any(moment(2:4))
        moment(2:4)=moment(1)*moment(2:4)/norm(moment(2:4));
    else
        moment(2:4)=[0,0,0];
    end
    if repartida(1) && any(repartida(2:4))
        repartida(2:4)=repartida(1)*repartida(2:4)/norm(repartida(2:4));
    else
        repartida(2:4)=[0,0,0];
    end
    if projectada(1) && any(projectada(2:4))
        projectada(2:4)=projectada(1)*projectada(2:4)/norm(projectada(2:4));
        vbarra=[1,0,0];
        vcarrega=projectada(2:4);
        vperp=cross(vbarra,vcarrega);
        vperp=cross(vperp,vcarrega);
        vperp=vperp/norm(vperp);
        dcx=abs(dot(vperp,vbarra));
    else
        projectada(2:4)=[0,0,0];
        dcx=0;
    end
end

%% CLCUL DELS POLINOMIS QUE DEFINEIXEN ELS DIAGRAMES D'AXILS I TALLANTS.

function T=calcul_tallant(forextrem,fpuntual,frepartida,fprojectada,long,dcx)
if forextrem || fpuntual(1) || frepartida(1) || fprojectada(1) % Si hi ha tallant en crea el diagrama.
    xT=[];
    % Defineix els trams del diagrama:
    if fpuntual(1)
        xT=fpuntual(2);
    end
    if frepartida(1)
        xT=[xT,frepartida(2),frepartida(3)];
    end
    if fprojectada(1)
        xT=[xT,fprojectada(2),fprojectada(3)];
    end
    xT=unique([0,xT,long]);
    T=zeros(numel(xT)-1,4);
    
    % Per a cada tram calcula els coeficients de la expressi analtica
    % del diagrama de tallants en y:
    for i=2:numel(xT)
        x0=xT(i-1);
        x1=xT(i);
        T(i-1,3:4)=[x0,x1];
        if fpuntual(1) && x1>fpuntual(2)
            T(i-1,1:2)=[0,-forextrem-fpuntual(1)];
        else
            T(i-1,1:2)=[0,-forextrem];
        end
        if frepartida(1) && x1>frepartida(2) && x1<=frepartida(3)
            T(i-1,1:2)=T(i-1,1:2)+[-frepartida(1),frepartida(1)*frepartida(2)];
        elseif frepartida(1) && x1>frepartida(3)
            T(i-1,1:2)=T(i-1,1:2)+[0,-frepartida(1)*(frepartida(3)-frepartida(2))];
        end
        if fprojectada(1) && x1>fprojectada(2) && x1<=fprojectada(3)
            T(i-1,1:2)=T(i-1,1:2)+[-fprojectada(1)*dcx,fprojectada(1)*fprojectada(2)*dcx];
        elseif fprojectada(1) && x1>fprojectada(3)
            T(i-1,1:2)=T(i-1,1:2)+[0,-fprojectada(1)*(fprojectada(3)-fprojectada(2))*dcx];
        end
    end
else
    T=[];
end
end

%% CLCUL DELS POLINOMIS QUE DEFINEIXEN ELS DIAGRAMES DE MOMENTS.

function M=calcul_moment(forextrem,momextrem,fmoment,fpuntual,frepartida,fprojectada,long,dcx,signe)
    if forextrem || momextrem || fmoment(1) || fpuntual(1) || frepartida(1) || fprojectada(1) % Si hi ha moments en eix z en crea el diagrama.
        xM=[];
        if fmoment(1)
            xM=fmoment(2);
        end
        if fpuntual(1)
            xM=[xM,fpuntual(2)];
        end
        if frepartida(1)
            xM=[xM,frepartida(2),frepartida(3)];
        end
        if fprojectada(1)
            xM=[xM,fprojectada(2),fprojectada(3)];
        end
        xM=unique([0,xM,long]);
        M=zeros(numel(xM)-1,5);
        for i=2:numel(xM)
            x0=xM(i-1); % Valor X a l'inici del tram.
            x1=xM(i); % Valor X al final del tram.
            M(i-1,4:5)=[x0,x1]; % Afegeix les dades d'X inicial i final a cada expressi analtica.
        
            % Expressi analtica del diagrama de moments segons els esforos.
            % En forma de vector amb els valors dels factors.
            if fmoment(1) && xM(i)>fmoment(2)
                M(i-1,1:3)=[0,forextrem*signe,-momextrem-fmoment(1)];
            else
                M(i-1,1:3)=[0,forextrem*signe,-momextrem];
            end
            if fpuntual(1) && x1>fpuntual(2)
                M(i-1,1:3)=M(i-1,1:3)+[0,fpuntual(1),-fpuntual(1)*fpuntual(2)]*signe;
            end
            if frepartida(1) && x1>frepartida(2) && x1<=frepartida(3)
                M(i-1,1:3)=M(i-1,1:3)+[frepartida(1)/2,-frepartida(1)*frepartida(2),frepartida(1)*frepartida(2)^2/2]*signe;
            elseif frepartida(1) && x1>frepartida(3)
                M(i-1,1:3)=M(i-1,1:3)+[0,frepartida(1)*(frepartida(3)-frepartida(2)),-frepartida(1)*(frepartida(3)-frepartida(2))*(frepartida(3)+frepartida(2))/2]*signe;
            end
            if fprojectada(1) && x1>fprojectada(2) && x1<=fprojectada(3)
                M(i-1,1:3)=M(i-1,1:3)+[fprojectada(1)*dcx/2,-fprojectada(1)*fprojectada(2)*dcx,fprojectada(1)*dcx*(fprojectada(2))^2/2]*signe;
            elseif fprojectada(1) && x1>fprojectada(3)
                M(i-1,1:3)=M(i-1,1:3)+[0,fprojectada(1)*(fprojectada(3)-fprojectada(2))*dcx,-fprojectada(1)*dcx*(fprojectada(3)-fprojectada(2))*(fprojectada(3)+fprojectada(2))/2]*signe;
            end
        end
    else
        M=[];
    end
end

%% CLCUL DELS POLINOMIS QUE DEFINEIXEN LES DEFORMACIONS.

function dx=calcul_dx(d0,N,EA,long)
    n=size(N,1); % Nmero de trams.
    if n
        dx=zeros(n,5);
        dx(:,4:5)=N(:,3:4);
        for i=1:n
            dx(i,1:3)=polyint(N(i,1:2)/EA);
            dx(i,3)=polyval([-dx(i,1:2),d0],dx(i,4));
            d0=polyval(dx(i,1:3),dx(i,5));
        end
    else
        dx=[0,0,d0,0,long];
    end
end

function [d,O,Oini,Ofin]=calcul_d(moviment,M,EI,long,signe)
    n=size(M,1); % Nmero de trams.
    if n
        O=zeros(n,6);
        d=zeros(n,7);
        O(:,5:6)=M(:,4:5);
        d(:,6:7)=M(:,4:5);
        O0=(moviment(2)-moviment(1))/long; % Valors inicials de gir de slid rgid i desplaament vertical.
        d0=moviment(1);
        for i=1:n
            x0=M(i,4);
            x1=M(i,5);
            O(i,1:4)=polyint(M(i,1:3)*signe/EI); % Integraci del diagrama de moments per obtenir l'expressi del gir.
            O(i,4)=polyval([-O(i,1:3),O0],x0); % Terme independent, per continuitat.
            O0=polyval(O(i,1:4),x1); % Gir al final del tram, ser l'inicial del tram segent. 
    
            d(i,1:5)=polyint(O(i,1:4)); % Integraci del gir per obtenir el desplaament vertical.
            d(i,5)=polyval([-d(i,1:4),d0],x0); % Terme independent, per continuitat.
            d0=polyval(d(i,1:5),x1); % Desplaament vertical al final del tram, ser l'inicial al tram segent.
        end
        O1=(moviment(2)-d0)/long; % Calcula el gir a l'extrem inferior imponent la condicio de contorn del desplaament vertical conegut a l'extrem superior.
        O(:,4)=O(:,4)+ones(n,1)*O1; % Corregeix les expressions de gir i desplaaments afegint el gir inicial calculat.
        d(:,4)=d(:,4)+ones(n,1)*O1;
        Oini=polyval(O(i,1:4),0); % Gir inicial.
        Ofin=polyval(O(i,1:4),long); % Gir final.
    elseif any(moviment)
        Oini=(moviment(2)-moviment(1))/long;
        d=[0,0,0,Oini,moviment(1),0,long];
        O=[0,0,0,Oini,0,long];
        Ofin=Oini;
    else
        d=[];
        O=[];
        Oini=0;
        Ofin=0;
    end
end

%% RECERCA DE ZEROS ALS POLINOMIS.

function x0=zero(P)
    [n,l]=size(P);
    x0=[];
    for i=1:n % Per a tots els trams comproba les arrels dels polinomis.
        x=roots(P(i,1:l-2)); % Arrels dels polinomis que defineixen els trams.
        for j=1:size(x,1); % Per a totes les arrels calculades...
            if isreal(x(j)) && x(j)>=P(i,l-1) && x(j)<=P(i,l) % Si l'arrel s un nombre real i est compresa entre els lmits del tram.
                x0=[x0,x(j)]; % Afegeix l'arrel a la llista.
            end
        end
    end
    
    for i=2:n % Comproba els lmits entre trams.
        if polyval(P(i-1,1:l-2),P(i-1,l)) * polyval(P(i,1:l-2),P(i,l-1)) < 0 % Si el valor final del tram anterior i el valor incial del tram actual sn de signe diferent...
            x0=[x0,P(i,l-1)]; % Afegeix el lmit entre els trams a la llista.
        end
    end
    x0=unique(x0); % Ordena i elimina resultats repetits.
end

%% DIBUIXA EL DIAGRAMA.

function [hg,miny,maxy]=dibudiag(txt,P,P0,xm,sep,long,detall,signe)
    if ~isempty(P)
        hg=hgtransform;
        miny=[0,Inf];
        maxy=[0,-Inf];
        [n,l]=size(P);
        xd=0;
        yd=0;
        for i=1:n % En cada tram on es defineix el diagrama...
            if l>4 && any(P(i,1:l-4)) % Si el diagrama s una corba defineix una srie de punts intermitjos per definirla.
                x=[P(i,l-1):sep:P(i,l)];
            else
                x=[P(i,l-1),P(i,l)]; % Si el diagrama s una recta en defineix els punts inicial i final del tram.
            end
            for j=1:size(xm,1) % Si dins del tram hi ha un candidat a mnim o mxim l'afegeix als punts perque sigui evaluat.
                if xm(j)>P(i,l-1) && xm(j)<P(i,l)
                    x=[x,xm(j)];
                end
            end
            x=unique(x); % Ordena els punts i en descarta els repetits.
            y=polyval(P(i,1:l-2),x); % Evalua els punts.
            xd=[xd,x]; % Afegeix els punts al diagrama.
            yd=[yd,y];
            [miny(2),k]=min([miny(2),y]); % Comproba si el mnim i mxim del tram sn els absoluts.
            if k~=1
                miny(1)=x(k-1);
            end
            [maxy(2),k]=max([maxy(2),y]);
            if k~=1
                maxy(1)=x(k-1);
            end
        end
        xd=[xd,P(n,l)]; % Uneix el punt final del diagrama amb la barra.
        yd=[yd,0];
        line(xd,-yd*signe,'Linewidth',2,'Parent',hg,'HitTest','off','Clipping','on') % Dibuixa el diagrama.
        
        if detall
            if miny(1)~=0 && miny(1)~=long
                scatter(miny(1),-miny(2)*signe,100,'+','k','Parent',hg) % Marca el mnim absolut.
                txtdiag(miny(1),miny(2),txt,long,hg,signe)
            end
            if maxy(1)~=0 && maxy(1)~=long && maxy(1)~=miny(1)
                scatter(maxy(1),-maxy(2)*signe,100,'+','k','Parent',hg) % Marca el mxim absolut.
                txtdiag(maxy(1),maxy(2),txt,long,hg,signe)
            end
            txtdiag(0,yd(2),txt,long,hg,signe) % Moment inicial.
            txtdiag(long,yd(length(yd)-1),txt,long,hg,signe) % Moment final.
                for i=1:size(P0,2) % Si dins del tram hi han ceros.
                    txtdiag(P0(i),0,txt,long,hg,signe) % Marca els ceros.
                end
        end
    else
        hg=[];
        miny=[0,0];
        maxy=[0,0];
    end
end

function txtdiag(x,y,txt,long,hg,signe)
    if x>long/2
        halin='right';
    else
        halin='left';
    end
    if y>0
        valin='top';
    else
        valin='bottom';
    end
    text(x,-y*signe,[txt,'(x=',num2str(x),')=',num2str(y)],'VerticalAlignment',valin,'HorizontalAlignment',halin,'Parent',hg);
end

%% DIBUIXA LA DEFORMADA.

function [hg,dxmax,dymax,dzmax]=dibudef(dx,dy,dz,My0,Mz0,Oy0,Oz0,Oyini,Oyfin,Ozini,Ozfin,sep,long,detall)
    hg=hgtransform;

    [nx,lx]=size(dx);
    [ny,ly]=size(dy);
    [nz,lz]=size(dz);

    if ny || nz %si hi ha deformaci transversal.
        t=[0,dx(:,5)']; % Defineix els trams.
        if ny
            t=[t,dy(:,7)'];
        end
        if nz
            t=[t,dz(:,7)'];
        end
        t=unique(t);
        
        ix=1;
        iy=1;
        iz=1;
        xt=[];
        e=[];
        s=[];
        y=[];
        z=[];

        for i=2:size(t,2)
            xinfly=[];
            xinflz=[];
            xmaxy=[];
            xmaxz=[];
            x=[t(i-1):sep:t(i),t(i)];
            for j=1:size(My0,2)
                if My0(j)>t(i-1) && My0(j)<t(i)
                    xinfly=[xinfly,My0(j)];
                end
            end
            for j=1:size(Mz0,2)
                if Mz0(j)>t(i-1) && Mz0(j)<t(i)
                    xinflz=[xinflz,Mz0(j)];
                end
            end
            for j=1:size(Oy0,2)
                if Oy0(j)>t(i-1) && Oy0(j)<t(i)
                    xmaxz=[xmaxz,Oy0(j)];
                end
            end
            for j=1:size(Oz0,2)
                if Oz0(j)>t(i-1) && Oz0(j)<t(i)
                    xmaxy=[xmaxy,Oz0(j)];
                end
            end
            
            x=unique([x,xinfly,xinflz,xmaxy,xmaxz]);
            xt=[xt,x];
            e=[e,polyval(dx(ix,1:3),x)]; % Deformaci unitria en x.
            s=[xt+e]; % Deformada en x.
            while t(i)>dx(ix,5)
                ix=ix+1;
            end
            if ny
                while t(i)>dy(iy,7)
                    iy=iy+1;
                end
                y=[y,polyval(dy(iy,1:5),x)]; % Deformada en y.
            else
                y=[y,x*0];
            end
            if nz
                while t(i)>dz(iz,7)
                    iz=iz+1;
                end
                z=[z,polyval(dz(iz,1:5),x)]; % Deformada en z.
            else
                z=[z,x*0];
            end

        end
    else
        xt=[0,dx(nx,5)];
        e=[polyval(dx(1,1:3),0),polyval(dx(nx,1:3),dx(nx,5))];
        s=xt+e;
        y=[0,0];
        z=[0,0];
    end
    defx.x=xt;
    defx.e=e;
    line(s,y,z,'Linewidth',2,'Parent',hg,'UserData',defx,'HitTest','off','Clipping','on') % Dibuixa la deformada.

    [xm,imx]=max(abs(s)); % Desplaament en x mxim.
    dxmax=[xt(imx),e(imx)];
    [xm,imy]=max(abs(y)); % Desplaament en y mxim.
    dymax=[xt(imy),y(imy)];
    [xm,imz]=max(abs(z)); % Desplaament en z mxim.
    dzmax=[xt(imz),z(imz)];
    
    if detall % Si s'ha cridat la funci amb l'opci de mostrar detalls numerics i marques.
        % Faig servir un petit truc per mostrar correctament els scatter3.
        % Matlab no els colloca correctament si s'encapsulen directament
        % en un hgtransform, el que faig s extreure els objectes que cont
        % l'scatter3 i ficar-los dins del hgtransform en comptes de ficar
        % el scatter3 dins del hgtransform. D'aquesta manera funciona.
        
        l=length(xt);
        set(get(scatter3(s(imy),y(imy),z(imy),100,'+','k'),'Children'),'Parent',hg,'UserData','dy') % Marca dy mx.
        for i=1:length(Mz0)
            imoz=find(xt==Mz0(i)); % Punt d'inflexi.
            if imoz(1) ~=1 && imoz(length(imoz)) ~=l % Si el punt d'inflexi no est a l'inici o al final.
                set(get(scatter3(s(imoz),y(imoz),z(imoz),100,'o','k'),'Children'),'Parent',hg,'UserData','dy') % Marca inflexi gir en z.
            end
        end
        if imy ~=1 && imy~=l % Si el desplaament maxim en y no est a l'inici ni al final...
            txt={['dx(x=',num2str(xt(imy)),')=',num2str(e(imy))];['dy(x=',num2str(xt(imy)),')=',num2str(y(imy))];['z(x=',num2str(xt(imy)),')=0']}; % Texte desplaament mxim en y.
            txtdef(s(imy),y(imy),z(imy),txt,'dy',long,hg)
        end
        txt={['dx(x=0)=',num2str(e(1))];['dy(x=0)=',num2str(y(1))];['z(x=0)=',num2str(Ozini)]}; % Texte desplaament en y a l'inici.
        txtdef(s(1),y(1),z(1),txt,'dy',long,hg)
        txt={['dx(x=',num2str(long),')=',num2str(e(l))];['dy(x=',num2str(long),')=',num2str(y(l))];['z(x=',num2str(long),')=',num2str(Ozfin)]}; % Texte desplaament en y al final.
        txtdef(s(l),y(l),z(l),txt,'dy',long,hg)

        set(get(scatter3(s(imz),y(imz),z(imz),100,'+','k'),'Children'),'Parent',hg,'UserData','dz') % Marca dz mx.
        for i=1:length(My0)
            imoy=find(xt==My0(i)); % Punt d'inflexi.
            if imoy(1) ~=1 && imoy(length(imoy)) ~=l % Si el punt d'inflexi no est a l'inici o al final.
                set(get(scatter3(s(imoy),y(imoy),z(imoy),100,'o','k'),'Children'),'Parent',hg,'UserData','dz') % Marca inflexi gir en y.
            end
        end
        if imz ~=1 && imz~=l % Si el desplaament maxim en z no est a l'inici ni al final...
            txt={['dx(x=',num2str(xt(imz)),')=',num2str(e(imz))];['dz(x=',num2str(xt(imz)),')=',num2str(z(imz))];['y(x=',num2str(xt(imz)),')=0']}; % Texte desplaament mxim en z.
            txtdef(s(imz),y(imz),z(imz),txt,'dz',long,hg)
        end
        txt={['dx(x=0)=',num2str(e(1))];['dz(x=0)=',num2str(z(1))];['y(x=0)=',num2str(Oyini)]}; % Texte desplaament en z a l'inici.
        txtdef(s(1),y(1),z(1),txt,'dz',long,hg)
        txt={['dx(x=',num2str(long),')=',num2str(e(l))];['dz(x=',num2str(long),')=',num2str(z(l))];['y(x=',num2str(long),')=',num2str(Oyfin)]}; % Texte desplaament en z al final.
        txtdef(s(l),y(l),z(l),txt,'dz',long,hg)
    end
end

function txtdef(x,y,z,txt,marca,long,hg)
    if x>long/2
        halin='right';
    else
        halin='left';
    end
    if y>0
        valin='bottom';
    else
        valin='top';
    end
    text(x,y,z,txt,'VerticalAlignment',valin,'HorizontalAlignment',halin,'Parent',hg,'UserData',marca);
end


%% CONVERTEIX ELS POLINOMIS EN TEXTE.

function txt=equac(polinomi)
    txt=[];
    [n,l]=size(polinomi);

    if n~=1 || any(polinomi(1,1:l-2))
        t=[];
        for i=1:n
            t=strcat(' (x=',num2str(polinomi(i,l-1)),'...',num2str(polinomi(i,l)),') =');
            if ~any(polinomi(i,1:l-2))
                t=strcat(t,' 0');
            else
                for j=1:l-2
                    t2=[];
                    if polinomi(i,j)
                        if j==l-2
                            t2=txtcoef(polinomi(i,j));
                        elseif polinomi(i,j)==1
                            t2=' + x';
                        elseif polinomi(i,j)==-1
                            t2=' - x';
                        else
                            t2=strcat(txtcoef(polinomi(i,j)),'*x');
                        end
                        if j<l-3
                            t2=strcat(t2,'^',num2str(l-j-2));
                        end
                    end
                    t=strcat(t,t2);
                end
            end
            txt=strcat(txt,t);
        end
    end
    txt={txt};
end

function txt=txtcoef(coef)
    if coef>0
        txt=[' + ',num2str(coef)];
    elseif coef<0
        txt=[' - ',num2str(-coef)];
    else
        txt='';
    end
end

function txt=monta_texte(txt,txt1,txt2,varargin)
    if ~isempty(txt2{1})
        switch nargin
            case 3
                txt=[txt;strcat(txt1,txt2)];
            case 4
                max=varargin{1};
                txtmax=strcat(txt1,'max =  ',txt1,'(x= ',num2str(max(1)),') =  ',num2str(max(2)));
                txt=[txt;strcat(txt1,txt2);txtmax];
            otherwise
                min=varargin{1};
                max=varargin{2};
                cero=varargin{3};
                txtmin=strcat(txt1,'min =  ',txt1,'(x= ',num2str(min(1)),') =  ',num2str(min(2)));
                txtmax=strcat(txt1,'max =  ',txt1,'(x= ',num2str(max(1)),') =  ',num2str(max(2)));
                txt=[txt;strcat(txt1,txt2);txtmin;txtmax];
                if size(cero,2)
                    txtkk=strcat(txt1,' = 0 en x = ');
                    txtcero='';
                    for i=1:size(cero,2)
                        txtcero=strcat(txtcero,txtkk,num2str(cero(i)));
                        txtkk=' ; x = ';
                    end
                    txt=[txt;txtcero];
                end
        end
    end
end