% Funci dibuix_projectada per a Clcul Matricial d'Estructures 2 (CME2).
%
% Per Sergio Guerrero Miralles, darrera versi 13 d'octubre de 2.012.
%
% Crea ojectes grfics del tipus hgtransform amb la representaci de les
% crregues projectades sobre la barra.
% Retorna el handle, per a collocar i escalar a la grfica del programa.
% Les crregues projectades sempre es representen per sobre de la barra.
%
% Entrada:
% - n: nmero de barra.
% - cordinf: coordenada del punt de la barra on comena la crrega.
% - cordsup: coordenada del punt de la barra on acaba la crrega .
% - projectada: vector de la crrega projectada en format [mdul,component x,component y,component z].
% - s: factor d'escala.
% - fontcarregues: tipus de font per als textes de les crregues.
% - tamanyfontcarregues: tamany dels textos.
%
% Sortida:
% - hgproj: handle de l'objecte hgtransform que representa la carrega projectada.

%% FUNCI PRINCIPAL:

function hgproj = dibuix_projectada(n,cordinf,cordsup,projectada,s,fontcarregues,tamanyfontcarregues,qmax)

if projectada(1) && any(projectada(2:4)) % Si hi ha definida corrctament la crrega projectada.
    
    projectada=projectada*projectada(1)/abs(projectada(1));
    vbarra=cordsup-cordinf; % Vector direcci de la barra.
    vcarrega=projectada(2:4); % Vector direcci de la crrega.
    prodesc=dot(vbarra/norm(vbarra),vcarrega/norm(vcarrega)); % Producte escalar dels vectors direcci de la barra i de la carrega.
    
    if any(vbarra) && ~(abs(prodesc)==1) % Si les direccions de la barra i la crrega no son paralleles...
        if prodesc>0 % Si el dibuix de la crrega es crea amb la barra intercanvia els extrems per evitar-ho.
            cordsup=cordinf;
            vbarra=-vbarra;
        end
        vperp=cross(vbarra,vcarrega); % Calcula un vector perpendicular a la barra i a la direcci de la crrega.
        vperp=cross(vperp,vcarrega); % Calcula un vector perpendicular a l'anterior i a la direcci de la crrega, forma un triangle al mateix plnol amb la crrega i la barra, s el vector projecci de la carrega sobre la barra.
        vperp=vperp/norm(vperp);
        lcarrega=dot(vperp,vbarra); % Longitud de la crrega.
        cordinf=cordsup-vperp*lcarrega; % Coordenada inferior de l'extrem de la projecci de la crrega.
        
        hgproj=hgtransform('HitTestArea','on','HitTest','off','Clipping','on','UserData',n); % Crea l'objecte hgtransform.
        sep=(-0.1-0.5*projectada(1)/qmax)*(s*projectada(2:4)/norm(projectada(2:4))); % Gruix de la crrega en forma de vector.
        postxt=dibuixa_linies(hgproj,sep,cordinf,cordsup); % Dibuixa les lnies que uneixen les fletxetes.
        dibuixa_fletxetes(lcarrega,cordinf,cordsup,vcarrega,hgproj,s,norm(sep)/s) % Dibuixa les fletxetes.
        dibuixa_texte(postxt,projectada,fontcarregues,tamanyfontcarregues,hgproj) % Collocaci del texte.
    else
        hgproj=[]; % Si la barra i la direcci de la crrega sn paralleles no hi ha crrega i no es dibuixa.
    end
else
    hgproj=[]; % Si la crrega no est definida corrctament no es dibuixa.
end
end

%% DIBUIX DE LNIES, FLETXES I TEXTE:

function lininf=dibuixa_linies(handle,separacio,cordinf,cordsup) % Dibuixa dos lnies paralleles separades per un vector.
    lininf=cordinf+separacio; % Coordenades de l'extrem inferior de la lnea parallela.
    linsup=cordsup+separacio; % Coordenades de l'extrem inferior de la lnea parallela.
    line([lininf(1),linsup(1)],[lininf(2),linsup(2)],[lininf(3),linsup(3)],'color','k','LineWidth',1,'Parent',handle); % Lnea parallela a la projecci de la crrega.
    line([cordinf(1),cordsup(1)],[cordinf(2),cordsup(2)],[cordinf(3),cordsup(3)],'color','k','LineWidth',1,'Parent',handle); % Lnea de la projecci de la crrega.
end
function dibuixa_fletxetes(long,cordinf,cordsup,vector,handle,s,separacio) % Dibuixa les fletxetes entre dos punts.
    numfletxes=abs(round(long/(0.2*s))); % Calcula el nmero de fletxes, a ms de la inicial, que hi caben segons l'escala.
    if numfletxes < 1
        numfletxes=1; % Mnim 1 fletxa ms la inicial.
    end
    interval=(cordsup-cordinf)/numfletxes; % Separaci entre fletxes.
    m=calcul_matriu(vector); % Matriu orientaci de les fletxetes.
    for i=0:numfletxes % dibuixa les fletxetes.
        dibuixa_fletxa(handle,s,separacio,(cordinf+interval*i)',m)
    end
end
function dibuixa_fletxa(handle,s,separacio,pos,m)
    XYZl=m*[-separacio,-0.1;0,0;0,0]*s+[pos,pos]; % Lnia de la fletxa rotada, escalada i collocada.
    XYZp=m*[-0.1,0,-0.1;0.018,0,-0.018;0,0,0]*s+[pos,pos,pos]; % Punta de la fletxa rotada, escalada i collocada.
    line(XYZl(1,:),XYZl(2,:),XYZl(3,:),'color','k','LineWidth',1,'Parent',handle,'HitTest','off');
    patch(XYZp(1,:),XYZp(2,:),XYZp(3,:),'k','Parent',handle,'HitTest','off');
end
function dibuixa_texte(postxt,carrega,fontcarregues,tamanyfontcarregues,handle)
    if carrega(2)>0
        halin='right';
    else
        halin='left';
    end
    if carrega(3)>0;
        valin='top';
    else
        valin='bottom';
    end
    text(postxt(1),postxt(2),postxt(3),strcat('Qp=',num2str(carrega(1))),'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'VerticalAlignment',valin,'HorizontalAlignment',halin,'Parent',handle);
end

%% CLCUL DE LA MATRIU DE CANVI DE BASE:

function matriu_rotacio=calcul_matriu(vector)
    vector=vector/norm(vector); % Normaliza el vector.
    if abs(vector(3))==1 % Si el vector s parallel a l'eix z...
        vector3=[-vector(3),0,0];
        vector2=[0,1,0];
    else
        vector2=cross([0,0,1],vector); % Calcula dos vectors perpendiculars al primer i entre ells per crear la matriu de canvi de base.
        vector2=vector2/norm(vector2);
        vector3=cross(vector,vector2);
        vector3=vector3/norm(vector3);
    end
    matriu_rotacio=[vector',vector2',vector3'];
end