% Funci dibuix_repartida per a Clcul Matricial d'Estructures 2 (CME2).
%
% Per Sergio Guerrero Miralles, darrera versi 13 d'octubre de 2.012.
%
% Crea ojectes grfics del tipus hgtransform amb la representaci de les
% crregues repartides sobre la barra.
% Retorna el handle, per a collocar i escalar a la grfica del programa.
% Les crregues repartides sempre penjen de la barra.
%
% Entrada:
% - n: nmero de barra.
% - cordinf: coordenada del punt de la barra on comena la crrega.
% - cordsup: coordenada del punt de la barra on acaba la crrega .
% - repartida: vector de la crrega repartida en format [mdul,component x,component y,component z].
% - s: factor d'escala.
% - fontcarregues: tipus de font per als textes de les crregues.
% - tamanyfontcarregues: tamany dels textos.
%
% Sortida:
% - hgrep: handle de l'objecte hgtransform que representa la carrega repartida.

%% FUNCI PRINCIPAL:

function hgrep = dibuix_repartida(n,cordinf,cordsup,repartida,s,fontcarregues,tamanyfontcarregues,qmax)

if repartida(1) && any(repartida(2:4)) % Si hi ha definida corrctament la crrega repartida.
    repartida=repartida*repartida(1)/abs(repartida(1));
    hgrep=hgtransform('HitTestArea','on','HitTest','off','Clipping','on','UserData',n); % Crea l'objecte hgtransform.
    sep=(0.1+0.5*repartida(1)/qmax)*(s*repartida(2:4)/norm(repartida(2:4))); % Gruix de la crrega en forma de vector.
    postxt=dibuixa_linies(hgrep,sep,cordinf,cordsup); % Dibuixa les lnies que uneixen les fletxetes.
    dibuixa_fletxetes(norm(cordsup-cordinf),cordinf,cordsup,repartida(2:4),hgrep,s,norm(sep)/s) % Dibuixa les fletxetes.
    dibuixa_texte(postxt,repartida,fontcarregues,tamanyfontcarregues,hgrep) % Collocaci del texte.
else
    hgrep=[]; % Si la crrega no est definida corrctament no es dibuixa.
end
end

%% DIBUIX DE LNIES, FLETXES I TEXTE:

function lininf=dibuixa_linies(handle,separacio,cordinf,cordsup) % Dibuixa dos lnies paralleles separades per un vector.
    lininf=cordinf+separacio; % Coordenades de l'extrem inferior de la lnea parallela.
    linsup=cordsup+separacio; % Coordenades de l'extrem inferior de la lnea parallela.
    line([lininf(1),linsup(1)],[lininf(2),linsup(2)],[lininf(3),linsup(3)],'color','k','LineWidth',1,'Parent',handle); % Lnea parallela a la projecci de la crrega.
    line([cordinf(1),cordsup(1)],[cordinf(2),cordsup(2)],[cordinf(3),cordsup(3)],'color','k','LineWidth',1,'Parent',handle); % Lnea de la projecci de la crrega.
end
function dibuixa_fletxetes(long,cordinf,cordsup,vector,handle,s,separacio) % Dibuixa les fletxetes entre dos punts.
    numfletxes=abs(round(long/(0.2*s))); % Calcula el nmero de fletxes, a ms de la inicial, que hi caben segons l'escala.
    if numfletxes < 1
        numfletxes=1; % Mnim 1 fletxa ms la inicial.
    end
    interval=(cordsup-cordinf)/numfletxes; % Separaci entre fletxes.
    m=calcul_matriu(vector); % Matriu orientaci de les fletxetes.
    for i=0:numfletxes % dibuixa les fletxetes.
        dibuixa_fletxa(handle,s,(cordinf+interval*i)',m,separacio)
    end
end
function dibuixa_fletxa(handle,s,pos,m,separacio)
    XYZl=m*[separacio,0;0,0;0,0]*s+[pos,pos]; % Lnia de la fletxa rotada, escalada i collocada.
    XYZp=m*[separacio-0.1,separacio,separacio-0.1;0.018,0,-0.018;0,0,0]*s+[pos,pos,pos]; % Punta de la fletxa rotada, escalada i collocada.
    line(XYZl(1,:),XYZl(2,:),XYZl(3,:),'color','k','LineWidth',1,'Parent',handle,'HitTest','off');
    patch(XYZp(1,:),XYZp(2,:),XYZp(3,:),'k','Parent',handle,'HitTest','off');
end
function dibuixa_texte(postxt,carrega,fontcarregues,tamanyfontcarregues,handle)
    if carrega(2)>0
        halin='right';
    else
        halin='left';
    end
    if carrega(3)>0;
        valin='bottom';
    else
        valin='top';
    end
    text(postxt(1),postxt(2),postxt(3),strcat('Qr=',num2str(carrega(1))),'FontName',fontcarregues,'FontUnits','pixels','FontSize',tamanyfontcarregues,'VerticalAlignment',valin,'HorizontalAlignment',halin,'Parent',handle);
end


%% CLCUL DE LA MATRIU DE CANVI DE BASE:

function matriu_rotacio=calcul_matriu(vector)
    vector=vector/norm(vector); % Normaliza el vector.
    if abs(vector(3))==1 % Si el vector s parallel a l'eix z...
        vector3=[-vector(3),0,0];
        vector2=[0,1,0];
    else
        vector2=cross([0,0,1],vector); % Calcula dos vectors perpendiculars al primer i entre ells per crear la matriu de canvi de base.
        vector2=vector2/norm(vector2);
        vector3=cross(vector,vector2);
        vector3=vector3/norm(vector3);
    end
    matriu_rotacio=[vector',vector2',vector3'];
end